{*******************************************************}
{   VV_lib (TUpdQuery, TUpdStoredProc, TScrollQuery)    }
{                                                       }
{        Copyright (c) 1999                             }
{          Vladimir Kuznetsov  (kuzn_vv@mail.ru)        }
{*******************************************************}
unit FieldVals;

interface

uses SysUtils, Classes, Db, DBTables;

type

  TFieldsValuesItem = class
  public
    FieldName : string;
    DataType: TFieldType;
    FieldVal: Variant;
  end;

  TFieldsValues = class
  private
    FFldList : TList; // Item -> TFieldsValuesItem
    function getFieldValue(const FieldName: string) : variant;
    procedure FreeFFldList;
  public
    property FieldValue[const FieldName: string] : variant
      read getFieldValue; default;
    property FldList : TList read FFldList;
    procedure ReadCurrRecord(aDataSet: TDataSet);
    function getFieldValueItem(const FieldName: string) : TFieldsValuesItem;
    constructor Create;
    destructor Destroy; override;
  end;

  // list for Record Number
  TRecNoItemType = (UseNone, UseId, UseBookmarkstr);

  TRecNoItem = class
  public
    id : integer;
    recno: integer;
    bookmarkstr: string;
    constructor CreateId(aid, arecno : integer);
    constructor CreateBkmrk(abookmarkstr: string;  arecno : integer);
  end;

  TRecNoList = class
  private
    FRecNoList : TList; // Item -> TRecNoItem
    procedure FreeRecNoList;
  public
    RecNoItemType: TRecNoItemType;
    DefaultRecNoItemType: Boolean;
    procedure ClearRecNoList;
    procedure AddItem(RecNoItem: TRecNoItem);
    function getRecNoById(const id: integer) : integer;
    function getRecNoByBkmrk(const bookmarkstr: string) : integer;
    function getRecNoItemByRecNo(const RecNo: integer) : TRecNoItem;
    procedure Sort;
    constructor Create;
    destructor Destroy; override;
  end;

const
  vvLibRecNotLocated = 'Record don''t located!';
  vvLibId = 'Id';

implementation

constructor TFieldsValues.Create;
begin
  FFldList := TList.Create;
end;

procedure TFieldsValues.FreeFFldList;
var
  i : integer;
  FieldsValuesItem: TFieldsValuesItem;
begin
  for i:= 0 to FFldList.Count-1 do
    begin
    FieldsValuesItem := TFieldsValuesItem(FFldList.Items[i]);
    FieldsValuesItem.Free;
    end;
end;

destructor TFieldsValues.Destroy;
begin
  FreeFFldList;
  FFldList.Free;
  inherited Destroy;
end;

procedure TFieldsValues.ReadCurrRecord(aDataSet: TDataSet);
var
  i : integer;
  FieldsValuesItem: TFieldsValuesItem;
begin
  FreeFFldList;
  FFldList.Clear ;

  for i:= 0 to aDataSet.FieldCount-1 do
    begin
    FieldsValuesItem := TFieldsValuesItem.Create;
    FieldsValuesItem.FieldName := aDataSet.Fields[i].FieldName;
    FieldsValuesItem.DataType  := aDataSet.Fields[i].DataType;
    FieldsValuesItem.FieldVal := aDataSet.Fields[i].Value;
    FFldList.Add(FieldsValuesItem);
    end;
end;

function TFieldsValues.getFieldValueItem(const FieldName: string) : TFieldsValuesItem;
var
  i : integer;
  Match : Boolean;
begin
  result := nil;
  Match := False;
  for i:= 0 to FFldList.Count-1 do
    begin
    result := TFieldsValuesItem(FFldList.Items[i]);
    if result.FieldName=FieldName then
      begin
      Match := True;
      break;
      end
    end;

  if not Match then
    result := nil;
//    raise Exception.Create(' '+FieldName+'  ');
end;

function TFieldsValues.getFieldValue(const FieldName: string) : variant;
var
  FieldsValuesItem: TFieldsValuesItem;
begin
  result := Null;
  FieldsValuesItem := getFieldValueItem(FieldName);
  if FieldsValuesItem<>nil then
    result := FieldsValuesItem.FieldVal;
end;

//-------------------------------------------------------------------
//-------------------------------------------------------------------
constructor TRecNoItem.CreateId(aid, arecno : integer);
begin
  id := aid;
  recno := arecno;
end;

constructor TRecNoItem.CreateBkmrk(abookmarkstr: string;  arecno : integer);
begin
  bookmarkstr := abookmarkstr;
  recno := arecno;
end;

constructor TRecNoList.Create;
begin
  FRecNoList := TList.Create;
  DefaultRecNoItemType := True;
end;

destructor TRecNoList.Destroy;
begin
  FreeRecNoList;
  FRecNoList.Free;
  inherited Destroy;
end;

procedure TRecNoList.FreeRecNoList;
var
  i : integer;
  RecNoItem: TRecNoItem;
begin
  for i:= 0 to FRecNoList.Count-1 do
    begin
    RecNoItem := TRecNoItem(FRecNoList.Items[i]);
    RecNoItem.Free;
    end;
end;

procedure TRecNoList.ClearRecNoList;
begin
  FreeRecNoList;
  FRecNoList.Clear;
end;

procedure TRecNoList.AddItem(RecNoItem: TRecNoItem);
begin
  FRecNoList.Add(RecNoItem);
end;

function RecNoItemIdCompare(Item1, Item2: Pointer): Integer;
var
  RecNoItem1, RecNoItem2: TRecNoItem;
begin
  RecNoItem1 := TRecNoItem(Item1);
  RecNoItem2 := TRecNoItem(Item2);
  result := RecNoItem1.Id- RecNoItem2.Id;
end;

function RecNoItemBkmrkCompare(Item1, Item2: Pointer): Integer;
var
  RecNoItem1, RecNoItem2: TRecNoItem;
begin
  RecNoItem1 := TRecNoItem(Item1);
  RecNoItem2 := TRecNoItem(Item2);
  result := CompareStr(RecNoItem1.bookmarkstr, RecNoItem2.bookmarkstr);
end;

procedure TRecNoList.Sort;
begin
  if RecNoItemType = UseId then
    FRecNoList.Sort(RecNoItemIdCompare)
  else
    FRecNoList.Sort(RecNoItemBkmrkCompare);
end;

function TRecNoList.getRecNoItemByRecNo(const RecNo: integer) : TRecNoItem;
var
  i: integer;
  RecNoItem: TRecNoItem;
begin
  result := nil;
  for i:=0 to FRecNoList.Count - 1 do
    begin
    RecNoItem := TRecNoItem(FRecNoList.Items[I]);
    if RecNoItem.recno=RecNo then
      begin
      result := RecNoItem;
      Exit;
      end;
    end;
end;

function TRecNoList.getRecNoById(const id: integer) : integer;
var
  RecNoItem: TRecNoItem;
  L, H, I, C: Integer;
begin
  result := -1;
  L := 0;
  H := FRecNoList.Count - 1;
  while L <= H do
    begin
    I := (L + H) shr 1;
    RecNoItem := TRecNoItem(FRecNoList.Items[I]);
    C :=  RecNoItem.id - id;
    if C = 0 then
      begin
      result :=  RecNoItem.RecNo;
      break;
      end;
    if C < 0 then
      L := I + 1
    else
      begin
      H := I - 1;
      end;
    end;
end;

function TRecNoList.getRecNoByBkmrk(const bookmarkstr: string) : integer;
var
  RecNoItem: TRecNoItem;
  L, H, I, C: Integer;
begin
  result := -1;
  L := 0;
  H := FRecNoList.Count - 1;
  while L <= H do
    begin
    I := (L + H) shr 1;
    RecNoItem := TRecNoItem(FRecNoList.Items[I]);
    C :=  CompareStr(RecNoItem.bookmarkstr, bookmarkstr);
    if C = 0 then
      begin
      result :=  RecNoItem.RecNo;
      break;
      end;
    if C < 0 then
      L := I + 1
    else
      begin
      H := I - 1;
      end;
    end;
end;

end.
