{*******************************************************}
{  VV_lib (TUpdQuery, TUpdStoredProc, TScrollQuery)     }
{                                                       }
{        Copyright (c) 1999                             }
{          Vladimir Kuznetsov  (kuzn_vv@mail.ru)        }
{*******************************************************}
unit ScrlQuery;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, FieldVals;

type
  TScrollQuery = class(TQuery)
  private
    { Private declarations }
    FRecNoList : TRecNoList;
  protected
    { Protected declarations }
    function GetRecNo: Integer; override;
    procedure SetRecNo(Value: Integer); override;
    procedure DoAfterOpen; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function IsSequenced: Boolean; override;
    property RecNoList : TRecNoList read FRecNoList;
  published
    { Published declarations }
  end;

procedure Register;

implementation

constructor TScrollQuery.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRecNoList := TRecNoList.Create;
end;

destructor TScrollQuery.Destroy;
begin
  FRecNoList.Free;
  inherited Destroy;
end;

function TScrollQuery.IsSequenced: Boolean;
begin
  Result := (not Filtered);
end;

function TScrollQuery.GetRecNo : integer;
var
  aFld : TField;
begin
  result := -1;
  case FRecNoList.RecNoItemType of
    UseId:
      begin
      aFld := FindField('Id');
      if assigned(aFld) then
        result := FRecNoList.getRecNoById(aFld.AsInteger)
      else
        raise Exception.Create(' Id  !');
      end;
    UseBookmarkstr :
      result := FRecNoList.getRecNoByBkmrk(Bookmark);
    end;
end;

procedure TScrollQuery.SetRecNo(Value: Integer);
var
  RecNoItem : TRecNoItem;
  aFld : TField;
begin
  RecNoItem := FRecNoList.getRecNoItemByRecNo(Value);
  if RecNoItem=nil then Exit;
  case FRecNoList.RecNoItemType of
    UseId:
      begin
      aFld := FindField('Id');
      if assigned(aFld) then
        Locate('Id', RecNoItem.Id, [])
      else
        raise Exception.Create(' Id  !');
      end;
    UseBookmarkstr :
      Bookmark := RecNoItem.bookmarkstr;
    end;
end;


procedure TScrollQuery.DoAfterOpen;
var
  iRecNo : integer;
  IdFld : TField;
  idexists : Boolean;
begin
  if (FRecNoList.DefaultRecNoItemType) or
     (FRecNoList.RecNoItemType<>UseNone) then
    begin
    DisableControls;
    try
      IdFld := FindField('Id');
      idexists := assigned(IdFld);
      FRecNoList.ClearRecNoList;
      if FRecNoList.DefaultRecNoItemType then
        if idexists then
          FRecNoList.RecNoItemType:=UseId
        else
          FRecNoList.RecNoItemType:=UseBookmarkstr;

      FetchAll;
      First; iRecNo:= 1;
      while not EOF do
        begin
        case FRecNoList.RecNoItemType of
          UseId:
            if idexists then
              FRecNoList.AddItem(TRecNoItem.CreateId(FieldByName('Id').AsInteger, iRecNo))
            else
              raise Exception.Create(' Id  !');
          UseBookmarkstr :
            FRecNoList.AddItem(TRecNoItem.CreateBkmrk(Bookmark, iRecNo));
          end;
        Next; Inc(iRecNo);
        end;
      First;
      FRecNoList.Sort;
    finally
      EnableControls;
      End;
    end;

  inherited DoAfterOpen;
end;

procedure Register;
begin
  RegisterComponents('vv_lib', [TScrollQuery]);
end;

end.
