{*******************************************************}
{   VV_lib (TUpdQuery, TUpdStoredProc, TScrollQuery)    }
{                                                       }
{        Copyright (c) 1999                             }
{          Vladimir Kuznetsov  (kuzn_vv@mail.ru)        }
{*******************************************************}
unit UpdQuery;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, FieldVals, DsgnIntf;

type
  TPosTyp = (posSavedId, posLastId, posNextId);
  TUpdQueryGetCurrRecordEvent = procedure(aPos: TPosTyp {posSavedId,posLastId,posNextId};
                                          var id: integer; var ptr: pointer) of object;
  TUpdQueryRefreshEvent = procedure(id: integer; ptr: pointer) of object;


type
  TUpdQuery = class(TQuery)
  private
    { Private declarations }
    FSQLText: array[TUpdateKind] of TStrings;
    FcurrFieldsValues : TFieldsValues;
    FQuery : TQuery;
    FRecNoList : TRecNoList;
    FRecNoFieldName : string;
    FDontResync: Boolean;

    FOnGetCurrRecordEvent: TUpdQueryGetCurrRecordEvent;
    FOnRefreshEvent : TUpdQueryRefreshEvent;

    function GetSQLIndex(Index: Integer): TStrings;
    procedure SetSQLIndex(Index: Integer; Value: TStrings);
    procedure InternalRefreshPos(pos: TPosTyp);
    function GetvvRequestLive : Boolean;
    procedure SetvvRequestLive(val : Boolean);
  protected
    { Protected declarations }
    function GetCanModify: Boolean; override;
    procedure InternalEdit; override;
    procedure InternalPost; override;
    procedure InternalDelete; override;
    procedure InternalRefresh; override;
    function GetRecNo: Integer; override;
    procedure SetRecNo(Value: Integer); override;
    procedure DoAfterOpen; override;

    procedure inetrnalBindParams(params: TParams);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ExecSQL(UpdateKind: TUpdateKind);
    function IsSequenced: Boolean; override;
    property RecNoList : TRecNoList read FRecNoList;
    procedure Resync(Mode: TResyncMode); override;
  published
    { Published declarations }
    property ModifySQL : TStrings index 0 read GetSQLIndex write SetSQLIndex;
    property InsertSQL : TStrings index 1 read GetSQLIndex write SetSQLIndex;
    property DeleteSQL : TStrings index 2 read GetSQLIndex write SetSQLIndex;

    property OnGetCurrRecord: TUpdQueryGetCurrRecordEvent
      read FOnGetCurrRecordEvent write FOnGetCurrRecordEvent;

    property OnRefresh: TUpdQueryRefreshEvent
      read FOnRefreshEvent write FOnRefreshEvent;

    property RequestLive : Boolean read GetvvRequestLive write SetvvRequestLive default False;
    property RecNoFieldName: string read FRecNoFieldName write FRecNoFieldName;
  end;

procedure Register;

implementation

type THackDataSet = class(TDataSet) end;

constructor TUpdQuery.Create(AOwner: TComponent);
var
  UpdateKind: TUpdateKind;
begin
  inherited Create(AOwner);
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
    FSQLText[UpdateKind] := TStringList.Create;
  FcurrFieldsValues := TFieldsValues.Create;
  FRecNoList := TRecNoList.Create;
end;

destructor TUpdQuery.Destroy;
var
  UpdateKind: TUpdateKind;
begin
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
    FSQLText[UpdateKind].Free;
  FcurrFieldsValues.Free;
  FRecNoList.Free;
  inherited Destroy;
end;

function TUpdQuery.GetvvRequestLive : Boolean;
begin
  result := False;
end;

procedure TUpdQuery.SetvvRequestLive(val : Boolean);
begin
  if val=True then
    raise Exception.Create('=  RequestLive=True !');
end;

function TUpdQuery.GetCanModify: Boolean;
begin
  result := True;
end;

function TUpdQuery.GetSQLIndex(Index: Integer): TStrings;
begin
  Result := FSQLText[TUpdateKind(Index)];
end;

procedure TUpdQuery.SetSQLIndex(Index: Integer; Value: TStrings);
begin
  FSQLText[TUpdateKind(Index)].Assign(Value);
end;

procedure TUpdQuery.inetrnalBindParams(Params: TParams);
var
  i, j : integer;
  Old: Boolean;
  Param, SfPar: TParam;
  PName: string;
  FieldsValuesItem: TFieldsValuesItem;
  Field: TField;
  Value: Variant;
begin
  for i:=0 to params.Count-1 do
    begin
    Param := Params[i];
    PName := Param.Name;
    Old := CompareText(Copy(PName, 1, 4), 'OLD_') = 0;
    if Old then
      begin
      // OLD Field values
      System.Delete(PName, 1, 4);
      FieldsValuesItem := FcurrFieldsValues.getFieldValueItem(PName);
      if assigned(FieldsValuesItem) then
        begin
        Param.DataType := FieldsValuesItem.DataType;
        Param.Value := FieldsValuesItem.FieldVal;
        end;
      end
    else
      begin
      Field := FindField(PName);
      if Assigned(Field) then
        begin
        // NEW Field values
        Value := Field.Value;
        //if VarIsEmpty(Value) then Value := Field.OldValue;
        Param.AssignFieldValue(Field, Value);
        end
      else
        begin
        // Self Param values
        for j:=0 to Self.Params.Count-1 do
          begin
          SfPar := Self.Params[j];
          if SfPar.Name=PName then
            begin
            Param.Assign(SfPar);
            Break;
            end;
          end;
        end;
      end;
    end;
end;

procedure TUpdQuery.ExecSQL(UpdateKind: TUpdateKind);
begin
  if not assigned(FQuery) then
    begin
    FQuery := TQuery.Create(Self);
    FQuery.DatabaseName := Self.DatabaseName;
    end;

  FQuery.SQL.Clear;
  FQuery.SQL.AddStrings(FSQLText[UpdateKind]);
  FQuery.Prepare;
  inetrnalBindParams(FQuery.Params);
  FQuery.ExecSQL;

end;

procedure TUpdQuery.InternalEdit;
begin
  FcurrFieldsValues.ReadCurrRecord(Self); //   . 
  //
  if Trim(ModifySQL.Text)='' then
     raise Exception.Create('ModifySQL   ');
end;

procedure TUpdQuery.InternalPost;
var
  apos: TPosTyp;
begin
  if State = dsEdit then
    begin
    ExecSQL(ukModify);
    apos:= posSavedId;
    end
  else
    begin
    ExecSQL(ukInsert);
    apos:= posLastId;
    end;

  InternalRefreshPos(apos);

  FDontResync := True;  //D5 sp1
  //DoAfterPost;
  //Abort;
end;

procedure TUpdQuery.InternalDelete;
begin
  FcurrFieldsValues.ReadCurrRecord(Self); //   . 
  ExecSQL(ukDelete);

  InternalRefreshPos(posNextId);

  FDontResync := True;  //D5 sp1
  //DoAfterDelete;
  //Abort;
end;

procedure TUpdQuery.InternalRefresh;
begin
  InternalRefreshPos(posSavedId);

  FDontResync := True; //D5 sp1
  //Abort
end;


procedure TUpdQuery.InternalRefreshPos(pos: TPosTyp);
var
  IdFld: TField;
  id : integer;
  idexists : Boolean;
  ptr: pointer;
begin
  idexists := False;
  id := 0;

  DisableControls;
  try
    IdFld := FindField('Id');
    if assigned(IdFld) then
      begin
      idexists := True;
      id := FieldByName('Id').AsInteger;
      end;

    if (pos=posNextId) and idexists then
      begin
      if not Eof then
        Next;
      if Eof then
        Prior;
      id := FieldByName('Id').AsInteger;
      end;

    if assigned(FOnGetCurrRecordEvent) then
      FOnGetCurrRecordEvent(pos, id, ptr);

    Close;
    Open;

    if (pos=posLastId) and idexists and
        not assigned(FOnGetCurrRecordEvent) then
      begin
      First;
      while not EOF do   //  max(Id)
        begin
        if FieldByName('Id').AsInteger>Id then
          Id:=FieldByName('Id').AsInteger;
        Next;
        end;
      end
    else
      begin
      FetchAll;
      First;
      end;

    if idexists and (id>0) then
    begin
      if LocateRecord('Id', id, [], True) then
        inherited Resync([rmExact, rmCenter])
      else
        ShowMessage(vvLibRecNotLocated);
    end;
    if assigned(FOnRefreshEvent) then
      FOnRefreshEvent(id, ptr);
  finally
    EnableControls;
  End;
  THackDataSet(Self).DataEvent(deDataSetChange, 0); // for D5
end;

function TUpdQuery.IsSequenced: Boolean;
begin
  Result := (not Filtered);
end;

function TUpdQuery.GetRecNo : integer;
var
  aFld : TField;
begin
  result := -1;

  if length(trim(RecNoFieldName))=0 then
    begin
    case FRecNoList.RecNoItemType of
      UseId:
        begin
        aFld := FindField('Id');
        if assigned(aFld) then
          result := FRecNoList.getRecNoById(aFld.AsInteger)
        else
          raise Exception.Create(' Id  !');
        end;
      UseBookmarkstr :
        result := FRecNoList.getRecNoByBkmrk(Bookmark);
      end;
    end
  else
    begin
    aFld := FindField(RecNoFieldName);
    if assigned(aFld) then
      result := aFld.AsInteger
    else
      raise Exception.Create(' '+RecNoFieldName+'  !');
    end;
end;

procedure TUpdQuery.SetRecNo(Value: Integer);
var
  RecNoItem : TRecNoItem;
  aFld : TField;
begin
  if length(trim(RecNoFieldName))=0 then
    begin
    RecNoItem := FRecNoList.getRecNoItemByRecNo(Value);
    if RecNoItem=nil then Exit;
    case FRecNoList.RecNoItemType of
      UseId:
        begin
        aFld := FindField('Id');
        if assigned(aFld) then
          Locate('Id', RecNoItem.Id, [])
        else
          raise Exception.Create(' Id  !');
        end;
      UseBookmarkstr :
        Bookmark := RecNoItem.bookmarkstr;
      end;
    end
  else
    begin
    Locate(RecNoFieldName, Value, []);
    end;
end;

procedure TUpdQuery.DoAfterOpen;
var
  iRecNo : integer;
  IdFld : TField;
  idexists : Boolean;
begin
  if length(trim(RecNoFieldName))=0 then
  begin
    DisableControls;
    try
      IdFld := FindField('Id');
      idexists := assigned(IdFld);
      FRecNoList.ClearRecNoList;
      if FRecNoList.DefaultRecNoItemType then
        if idexists then
          FRecNoList.RecNoItemType:=UseId
        else
          FRecNoList.RecNoItemType:=UseBookmarkstr;

      First; iRecNo:= 1;
      while not EOF do
        begin
        case FRecNoList.RecNoItemType of
          UseId:
            if idexists then
              FRecNoList.AddItem(TRecNoItem.CreateId(FieldByName('Id').AsInteger, iRecNo))
            else
              raise Exception.Create(' Id  !');
          UseBookmarkstr :
            FRecNoList.AddItem(TRecNoItem.CreateBkmrk(Bookmark, iRecNo));
          end;
        Next; Inc(iRecNo);
        end;
      First;
      FRecNoList.Sort;
    finally
      EnableControls;
    End;
  end;
  inherited DoAfterOpen;
end;

procedure TUpdQuery.Resync(Mode: TResyncMode);
begin
  If FDontResync then // Empty if Flag Set D5 sp1
    FDontResync := False
  else
    inherited;
end;


procedure Register;
begin
  RegisterComponents('vv_lib', [TUpdQuery]);
  RegisterPropertyEditor(TypeInfo(Boolean), TUpdQuery, 'RequestLive', nil);
end;

end.
