{*******************************************************}
{   VV_lib (TUpdQuery, TUpdStoredProc, TScrollQuery)    }
{                                                       }
{        Copyright (c) 1999                             }
{          Vladimir Kuznetsov  (kuzn_vv@mail.ru)        }
{*******************************************************}
unit vvMultiSelectCombo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ToolEdit, extctrls, RXCtrls, VCLUtils, buttons;

type
  TvvMultiSelectCombo = class(TCustomComboEdit)
  private
    FItems: TStrings;
    FValues: TStrings;
    FPrefMultiSel, FPrefAllSel: string;
    procedure SetItems(const Value: TStrings);
    procedure SetValues(const Value: TStrings);
    procedure ItemsChanged(Sender: TObject);
    function Get_Value: string;
    procedure SetChecked(Index: Integer; AChecked: Boolean);
    function GetChecked(Index: Integer): Boolean;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Loaded; override;
    function GetDefaultBitmap(var DestroyNeeded: Boolean): TBitmap; override;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMEnter(var Message: TMessage); message CM_ENTER;
    procedure CMHintShow(var Msg : TMessage); message CM_HINTSHOW;
    procedure PopupDropDown(DisableEdit: Boolean); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Button;
    property Value: String read Get_Value;
    property Checked[Index: Integer]: Boolean read GetChecked write SetChecked;
    procedure CheckAll;
  published
    { Published declarations }
    property Items: TStrings read FItems write SetItems;
    property Values: TStrings read FValues write SetValues;
    property PrefMultiSel: string read FPrefMultiSel write FPrefMultiSel;
    property PrefAllSel: string read FPrefAllSel write FPrefAllSel;

    property Alignment;
    property AutoSelect;
    property BorderStyle;
    property ButtonHint;
    property CharCase;
    property ClickKey;
    property Color;
    property Ctl3D;
    //property DirectInput;  Alvays False
    property DragCursor;
    property DragMode;
    property EditMask;
    property Enabled;
    property Font;
    property GlyphKind;
    { Ensure GlyphKind is published before Glyph and ButtonWidth }
    property Glyph;
    property ButtonWidth;
    property HideSelection;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property NumGlyphs;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    //property Text;
    property Visible;
    property OnButtonClick;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnContextPopup;
    property OnEndDock;
    property OnStartDock;
  end;

procedure Register;

implementation

{$R *.DCR}

procedure Register;
begin
  RegisterComponents('vv_lib', [TvvMultiSelectCombo]);
end;

Type
  TPopupCheckListBox = class(TPopupWindow)
  private
    FRxCheckListBox: TRxCheckListBox;
  protected
    procedure RxCheckListBoxClick(Sender:TObject);
    procedure RxCheckListBoxKeyUp(Sender:TObject;var Key:Word;Shift:TShiftState);
    procedure RxCheckListBoxExit(Sender: TObject);

    function GetValue: Variant; override;
    procedure SetValue(const Value: Variant); override;
    procedure SetItems;
  public
    constructor Create(AOwner: TComponent); override;
  end;


constructor TPopupCheckListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Height :=  200;
  Width :=  200;
  Color := clBtnFace;

  if (csDesigning in ComponentState) then Exit;

  FRxCheckListBox := TRxCheckListBox.Create(Self);
  with FRxCheckListBox do begin
    Parent := Self; //
    Align  := alClient;
    MultiSelect := True;
    OnExit  := RxCheckListBoxExit;
    OnClick := RxCheckListBoxClick;
    OnKeyUp := RxCheckListBoxKeyUp;
  end;

  FRxCheckListBox.IntegralHeight := True;
  AutoSize := True;

  ParentShowHint := False;
  ShowHint := False;
end;


function TPopupCheckListBox.GetValue: Variant;
var
  i: integer;
  sCheckedItems, sHint : string;
  bMulti, bAll: Boolean;
begin
  bMulti := False;  bAll := True;
  sCheckedItems:= '';  sHint := '';
  for i:=0 to FRxCheckListBox.Items.Count-1 do
    if FRxCheckListBox.Checked[i] then
    begin
      If sCheckedItems<>'' then
      begin
        bMulti := True;
        sCheckedItems := sCheckedItems + ', ';
        sHint := sHint + #10#13;
      end;
      sCheckedItems := sCheckedItems + FRxCheckListBox.Items[i];
      sHint := sHint + FRxCheckListBox.Items[i];
    end else
      bAll := False;

  TControl(Owner).Hint := sHint;

  if bAll then  sCheckedItems := TvvMultiSelectCombo(Owner).FPrefAllSel + sCheckedItems
  else if bMulti then  sCheckedItems := TvvMultiSelectCombo(Owner).FPrefMultiSel + sCheckedItems;
  result := sCheckedItems;
end;

procedure TPopupCheckListBox.SetValue(const Value: Variant);
begin
  //sDbg := 'SetValue';
end;


procedure TPopupCheckListBox.SetItems;
begin
  FRxCheckListBox.Items.Assign(TvvMultiSelectCombo(Owner).Items);
end;

procedure TPopupCheckListBox.RxCheckListBoxExit(Sender: TObject);
begin
  CloseUp(True);
end;

procedure TPopupCheckListBox.RxCheckListBoxClick(Sender: TObject);
var
  i: integer;
  ShiftState, CtrlState :SHORT;
begin
  FRxCheckListBox.Items.BeginUpdate;
  for i:=0 to FRxCheckListBox.Items.Count-1 do
   FRxCheckListBox.Checked[i]:=FRxCheckListBox.Selected[i];
  FRxCheckListBox.Items.EndUpdate;

  ShiftState := GetKeyState(VK_SHIFT);
  CtrlState := GetKeyState(VK_CONTROL);
  If ((CtrlState and $8000)>0) or
     ((ShiftState and $8000)>0) Then Exit;

  CloseUp(True); //ShowMessage('Close Up!')
end;

procedure TPopupCheckListBox.RxCheckListBoxKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
 If (Key=VK_SHIFT) or (Key=VK_CONTROL) then
    CloseUp(True); //ShowMessage('Close Up!')
end;


{ TvvMultiSelectCombo }

constructor TvvMultiSelectCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChanged;
  FValues := TStringList.Create;

  FPrefMultiSel := '{..} ';
  FPrefAllSel := '{} ';

  DirectInput := False;
  ControlState := ControlState + [csCreating];
  try
    FPopup := TPopupWindow(TPopupCheckListBox.Create(Self));
    TPopupWindow(FPopup).OnCloseUp := PopupCloseUp;
    GlyphKind := gkDefault; { force update }
  finally
    ControlState := ControlState - [csCreating];
  end;
end;

destructor TvvMultiSelectCombo.Destroy;
begin
  FItems.Free;
  FItems := nil;
  FValues.Free;
  FValues := nil;
  if FPopup <> nil then TPopupWindow(FPopup).OnCloseUp := nil;
  FPopup.Free;
  FPopup := nil;
  inherited Destroy;
end;

procedure TvvMultiSelectCombo.Loaded;
begin
  inherited Loaded;

  if not (csDesigning in ComponentState) then
  begin
    Text := '';
    TPopupCheckListBox(FPopup).SetItems;
  end;
end;

procedure TvvMultiSelectCombo.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
  if not (csDesigning in ComponentState) then
    TPopupCheckListBox(FPopup).SetItems;
end;

procedure TvvMultiSelectCombo.SetValues(const Value: TStrings);
begin
  FValues.Assign(Value);
end;

procedure TvvMultiSelectCombo.ItemsChanged(Sender: TObject);
begin
  if not (csDesigning in ComponentState) then
    TPopupCheckListBox(FPopup).SetItems;
end;

function TvvMultiSelectCombo.Get_Value: string;
var
  i: integer;
  sValuesItems, sValue : string;
begin
  sValuesItems:= '';
  for i:=0 to FItems.Count-1 do
    if GetChecked(i) then
    begin
      If i< FValues.Count then sValue := FValues[i]
      Else sValue := FItems[i];

      If sValuesItems<>'' then sValuesItems := sValuesItems + ', ';
      sValuesItems := sValuesItems + sValue;
    end;
  result := sValuesItems;
end;

procedure TvvMultiSelectCombo.SetChecked(Index: Integer; AChecked: Boolean);
begin
  TPopupCheckListBox(FPopup).FRxCheckListBox.Checked[Index] := AChecked;
  TPopupCheckListBox(FPopup).FRxCheckListBox.Selected[Index] := AChecked;
  Text := TPopupCheckListBox(FPopup).GetValue;
end;

function TvvMultiSelectCombo.GetChecked(Index: Integer): Boolean;
begin
  result := TPopupCheckListBox(FPopup).FRxCheckListBox.Checked[Index];
end;

procedure TvvMultiSelectCombo.CheckAll;
var
  i: integer;
  aRxCheckListBox: TRxCheckListBox;
begin
  aRxCheckListBox := TPopupCheckListBox(FPopup).FRxCheckListBox;
  aRxCheckListBox.Items.BeginUpdate;
  try
    for i:=0 to Items.Count-1 do
    begin
      aRxCheckListBox.Checked[i] := True;
      aRxCheckListBox.Selected[i] := True;
    end;
  finally
    aRxCheckListBox.Items.EndUpdate;
  end;
   Text := TPopupCheckListBox(FPopup).GetValue;
end;


const MultiSelBitmap: TBitmap = nil;

function TvvMultiSelectCombo.GetDefaultBitmap(var DestroyNeeded: Boolean): TBitmap;
begin
  DestroyNeeded := False;
  if MultiSelBitmap = nil then begin
    MultiSelBitmap := TBitmap.Create;
    MultiSelBitmap.Handle := LoadBitmap(hInstance, PChar(32738)); //PChar(32738)); //'FEDITBMP'
  end;
  Result := MultiSelBitmap;
end;

procedure TvvMultiSelectCombo.WMKillFocus(var Message: TWMKillFocus);
begin
 if (Message.FocusedWnd=TPopupCheckListBox(FPopup).FRxCheckListBox.Handle) or
    (not PopupVisible) then
    inherited;
end;

procedure TvvMultiSelectCombo.CMExit(var Message: TCMExit);
begin
  inherited;
  if (not (csDesigning in ComponentState)) then
  begin
    SelStart :=0;
    SelLength :=0;
  end;
end;

procedure TvvMultiSelectCombo.CMEnter(var Message: TMessage);
begin
  if (not (csDesigning in ComponentState)) then
  begin
    SelStart := Length(Text);
    SelLength := -Length(Text);
  end;
  inherited;
end;

procedure TvvMultiSelectCombo.CMHintShow(var Msg : TMessage);
var
  ptrHintInfo: PHintInfo;
begin
  ptrHintInfo := PHintInfo(Msg.LParam);
  if not PointInRect(ptrHintInfo^.CursorPos, Self.ClientRect) then
    begin
    //ptrHintInfo^.HintStr := ' !';
    Msg.Result := 1;
    end;
  //else
  //  ptrHintInfo^.HintStr := '!';
  inherited;
end;

procedure TvvMultiSelectCombo.PopupDropDown(DisableEdit: Boolean);
begin
  TPopupCheckListBox(FPopup).AutoSize := False;
  TPopupCheckListBox(FPopup).Width := Width;
  TPopupCheckListBox(FPopup).AutoSize := True;;
  inherited;
end;

initialization

finalization
  MultiSelBitmap.Free;

end.
