unit SmartDBmwEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  mwCustomEdit, DBmwEdit, mwKeyCmds, Menus, imglist, DsgnIntf;

type
  TFindReplaceType = (frsNone, frsFind, frsReplace);

type
  TSmartDBmwEdit = class(TDBmwEdit)
  private
    FFindDialog: TFindDialog;
    FReplaceDialog: TReplaceDialog;
    FPopupMenu: TPopupMenu;
    FPopupMenuImages: TCustomImageList;
    FPopupMenuFindImageIndex: TImageIndex;
    FPopupMenuReplaceImageIndex: TImageIndex;
    FPopupMenuNextSrchImageIndex: TImageIndex;
    FLastFindReplaceType : TFindReplaceType;

    procedure DoFindText(Sender: TObject);
    procedure DoReplaceText(Sender: TObject);
    procedure SetPopupMenuImages(Value: TCustomImageList);
    procedure DoDropFiles(Sender: TObject; X, Y: Integer; Files: TStrings);
  protected
    { Protected declarations }
    procedure GoFind(Sender: TObject);
    procedure GoReplace(Sender: TObject);
    procedure GoNextSearch(Sender: TObject);
    procedure GoCopyClipboardEx(Sender: TObject);
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property PopupMenuImages: TCustomImageList
      read FPopupMenuImages write SetPopupMenuImages;
    property PopupMenuFindImageIndex: TImageIndex
      read FPopupMenuFindImageIndex write FPopupMenuFindImageIndex default -1;
    property PopupMenuReplaceImageIndex: TImageIndex
      read FPopupMenuReplaceImageIndex write FPopupMenuReplaceImageIndex default -1;
    property PopupMenuNextSrchImageIndex: TImageIndex
      read FPopupMenuNextSrchImageIndex write FPopupMenuNextSrchImageIndex default -1;
  end;



procedure Register;

implementation


constructor TSmartDBmwEdit.Create(AOwner: TComponent);
var
  Item : TMenuItem;
begin
  inherited Create(AOwner);
  FLastFindReplaceType := frsNone;
  ClipboardFormats := [cfRTF, cfHTML];
  FPopupMenuFindImageIndex := -1;
  FPopupMenuReplaceImageIndex := -1;
  FPopupMenuNextSrchImageIndex := -1;

  FPopupMenu := TPopupMenu.Create(Self);

  Item := TMenuItem.Create( FPopupMenu );
  Item.Caption :=  ' ';
  Item.ShortCut := ShortCut( Word( 'F' ), [ssCtrl] );
  Item.OnClick := GoFind;
  FPopupMenu.Items.Add(Item);

  Item := TMenuItem.Create( FPopupMenu );
  Item.Caption :=  ' ';
  Item.ShortCut := ShortCut( Word( 'R' ), [ssCtrl] );
  Item.OnClick := GoReplace;
  FPopupMenu.Items.Add(Item);

  Item := TMenuItem.Create( FPopupMenu );
  Item.Caption :=  '  ';
  Item.ShortCut := ShortCut( VK_F3, [] );
  Item.OnClick := GoNextSearch;
  FPopupMenu.Items.Add(Item);

  Item := TMenuItem.Create( FPopupMenu );
  Item.Caption :=  '-';
  FPopupMenu.Items.Add(Item);

  Item := TMenuItem.Create( FPopupMenu );
  Item.Caption :=  '   (HTML, RTF)';
  Item.OnClick := GoCopyClipboardEx;
  FPopupMenu.Items.Add(Item);

  PopupMenu := FPopupMenu;
end;

procedure TSmartDBmwEdit.Loaded;
begin
  inherited Loaded;
  If Assigned(PopupMenuImages) then
  begin
    PopupMenu.Images := FPopupMenuImages;
    FPopupMenu.Items[0].ImageIndex := FPopupMenuFindImageIndex;
    FPopupMenu.Items[1].ImageIndex := FPopupMenuReplaceImageIndex;
    FPopupMenu.Items[2].ImageIndex := FPopupMenuNextSrchImageIndex;
  end;
  OnDropFiles := DoDropFiles;
end;

procedure TSmartDBmwEdit.SetPopupMenuImages(Value: TCustomImageList);
begin
  //if FPopupMenuImages <> nil then FPopupMenuImages.UnRegisterChanges(FImageChangeLink);
  FPopupMenuImages := Value;
  if FPopupMenuImages <> nil then
  begin
    //FPopupMenuImages.RegisterChanges(FImageChangeLink);
    FPopupMenuImages.FreeNotification(Self);
  end;
  //UpdateItems;
end;


destructor TSmartDBmwEdit.Destroy;
begin
  If Assigned(FFindDialog) Then
    FFindDialog.FreeOnRelease;
  If Assigned(FReplaceDialog) Then
    FReplaceDialog.Free;
  FPopupMenu.Free;  
  inherited;
end;


procedure TSmartDBmwEdit.GoFind(Sender: TObject);
begin
  If not Assigned(FFindDialog) Then
  begin
    FFindDialog := TFindDialog.Create(nil);
    FFindDialog.OnFind := DoFindText;
  end;
  FLastFindReplaceType := frsFind;
  FFindDialog.Execute;
end;

procedure TSmartDBmwEdit.GoReplace(Sender: TObject);
begin
  If not Assigned(FReplaceDialog) Then
  begin
    FReplaceDialog := TReplaceDialog.Create(nil);
    FReplaceDialog.OnReplace := DoReplaceText;
    FReplaceDialog.OnFind := DoFindText;
  end;
  FLastFindReplaceType := frsReplace;
  FReplaceDialog.Execute;
end;


procedure TSmartDBmwEdit.DoFindText(Sender: TObject);
var rOptions: TmwSearchOptions;
    dlg: TFindDialog;
    sSearch: string;
begin
  if Sender = FReplaceDialog then dlg := FReplaceDialog
                             else dlg := FFindDialog;
  sSearch := dlg.FindText;
  if Length(sSearch) = 0 then begin
    Beep;
    ShowMessage('Can''t search for empty text!');
  end else begin
    rOptions := [];
    if not (frDown in dlg.Options) then
      Include(rOptions, mwsoBackwards);
    if frMatchCase in dlg.Options then
      Include(rOptions, mwsoMatchCase);
    if frWholeWord in dlg.Options then
      Include(rOptions, mwsoWholeWord);
    if Self.SearchReplace(sSearch, '', rOptions) = 0 then begin
      Beep;
      ShowMessage('SearchText ''' + sSearch + ''' not found!');
    end;
  end;
end;

procedure TSmartDBmwEdit.DoReplaceText(Sender: TObject);
var rOptions: TmwSearchOptions;
    sSearch: string;
    aChar: char;
    Data: pointer;
    Command: TmwEditorCommand;
begin
  aChar := #0;
  Data := nil;
  Command := ecPaste;
  sSearch := FReplaceDialog.FindText;
  if Length(sSearch) = 0 then begin
    Beep;
    ShowMessage('Can''t replace an empty text!');
  end else begin
    rOptions := [mwsoReplace];
    if frMatchCase in FReplaceDialog.Options then
      Include(rOptions, mwsoMatchCase);
    if frWholeWord in FReplaceDialog.Options then
      Include(rOptions, mwsoWholeWord);
    if frReplaceAll in FReplaceDialog.Options then
      Include(rOptions, mwsoReplaceAll);
    Self.ProcessCommand(Command, aChar, Data); // !!!!!!  for Enter in EditMode
    if Self.SearchReplace(sSearch, FReplaceDialog.ReplaceText, rOptions) = 0
    then begin
      Beep;
      ShowMessage('SearchText ''' + sSearch +
                                 ''' could not be replaced!');
    end;
  end;
end;

procedure TSmartDBmwEdit.GoNextSearch(Sender: TObject);
begin
  case FLastFindReplaceType of
  //frsNone:
  frsFind:     DoFindText(FFindDialog);
  frsReplace:  DoReplaceText(FReplaceDialog);
  end;
end;


function IsPointEq(p1, p2: TPoint): Boolean;
begin
  result := (p1.x=p2.x) and (p1.y=p2.y);
end;

procedure TSmartDBmwEdit.GoCopyClipboardEx(Sender: TObject);
var
  bSetSelectAll: Boolean;
  CaretPos: TPoint;
begin
  bSetSelectAll := False;
  If IsPointEq(BlockBegin,BlockEnd) Then
  begin
    bSetSelectAll := True;
    CaretPos := CaretXY;
    SelectAll;
  end;

  CopyToClipboardEx;

  If bSetSelectAll Then
  begin
    BlockEnd := BlockBegin;
    CaretXY := CaretPos;
  end;
end;

procedure TSmartDBmwEdit.DoDropFiles(Sender: TObject; X, Y: Integer;                //mh 1999-11-02
            Files: TStrings);
begin
  ComputeCaret(x,y);
  SelText := Files.Text;
end;



procedure Register;
begin
  RegisterComponents('vv_mwEdit', [TSmartDBmwEdit]);
  RegisterPropertyEditor(TypeInfo(TPopupMenu), TSmartDBmwEdit, 'PopupMenu', nil);
end;

end.
